package asmc

import (
	"strings"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/x86"
)

func (c *compiler) move(x asm.Instruction) {
	switch operands := x.Data.(type) {
	case *asm.RegisterNumber:
		c.code = x86.MoveRegisterNumber(c.code, operands.Register, operands.Number)

	case *asm.RegisterRegister:
		c.code = x86.MoveRegisterRegister(c.code, operands.Destination, operands.Source)

	case *asm.RegisterLabel:
		start := Address(len(c.code))
		c.code = x86.LoadAddress(c.code, operands.Register, 0x00_00_00_00)
		end := Address(len(c.code))
		position := end - 4
		opSize := position - start

		if strings.HasPrefix(operands.Label, "data ") {
			c.dataPointers = append(c.dataPointers, &pointer{
				Position: position,
				OpSize:   uint8(opSize),
				Size:     uint8(4),
				Resolve: func() Address {
					destination, exists := c.dataLabels[operands.Label]

					if !exists {
						panic("unknown label")
					}

					destination += c.dataStart - c.codeStart
					distance := destination - end
					return distance + 8
				},
			})
		} else {
			c.codePointers = append(c.codePointers, &pointer{
				Position: position,
				OpSize:   uint8(opSize),
				Size:     uint8(4),
				Resolve: func() Address {
					destination, exists := c.codeLabels[operands.Label]

					if !exists {
						panic("unknown label")
					}

					return destination - end
				},
			})
		}
	}
}
