package build

import (
	"path/filepath"
	"strings"

	"git.urbach.dev/cli/q/src/config"
)

// Executable returns the path to the executable.
func (build *Build) Executable() string {
	path, _ := filepath.Abs(build.Files[0])

	if strings.HasSuffix(path, ".q") {
		path = fromFileName(path)
	} else {
		path = fromDirectoryName(path)
	}

	if config.TargetOS == config.Windows {
		path += ".exe"
	}

	return path
}

// fromDirectoryName returns the executable path based on the directory name.
func fromDirectoryName(path string) string {
	return filepath.Join(path, filepath.Base(path))
}

// fromFileName returns the executable path based on the file name.
func fromFileName(path string) string {
	return filepath.Join(filepath.Dir(path), strings.TrimSuffix(filepath.Base(path), ".q"))
}
