package compiler

import (
	"bufio"
	"io"

	"git.urbach.dev/cli/q/src/config"
	"git.urbach.dev/cli/q/src/dll"
	"git.urbach.dev/cli/q/src/elf"
	"git.urbach.dev/cli/q/src/macho"
	"git.urbach.dev/cli/q/src/pe"
)

// Write writes the executable to the given writer.
func (r *Result) Write(writer io.Writer) error {
	return write(writer, r.Code, r.Data, r.DLLs)
}

// write writes an executable file to the given writer.
func write(writer io.Writer, code []byte, data []byte, dlls dll.List) error {
	buffer := bufio.NewWriter(writer)

	switch config.TargetOS {
	case config.Linux:
		elf.Write(buffer, code, data)
	case config.Mac:
		macho.Write(buffer, code, data)
	case config.Windows:
		pe.Write(buffer, code, data, dlls)
	}

	return buffer.Flush()
}
